@@ -71,22 +71,22 @@ static int bt_setdirection_linux(pcap_t *, pcap_direction_t);
 static int bt_stats_linux(pcap_t *, struct pcap_stat *);
 
 int 
-bt_platform_finddevs(pcap_if_t **alldevsp, char *err_str)
+bt_findalldevs(pcap_if_t **alldevsp, char *err_str)
 {
 	pcap_if_t *found_dev = *alldevsp;
 	struct hci_dev_list_req *dev_list;
 	struct hci_dev_req *dev_req;
 	int i, sock;
 	int ret = 0;
-	
+
 	sock  = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (sock < 0)
 	{
 		/* if bluetooth is not supported this this is not fatal*/ 
 		if (errno == EAFNOSUPPORT)
 			return 0;
-		snprintf(err_str, PCAP_ERRBUF_SIZE, "Can't open raw Bluetooth socket %d:%s",
-			errno, strerror(errno));
+		snprintf(err_str, PCAP_ERRBUF_SIZE,
+		    "Can't open raw Bluetooth socket: %s", strerror(errno));
 		return -1;
 	}
 
@@ -103,8 +103,9 @@ bt_platform_finddevs(pcap_if_t **alldevsp, char *err_str)
 
 	if (ioctl(sock, HCIGETDEVLIST, (void *) dev_list) < 0) 
 	{
-		snprintf(err_str, PCAP_ERRBUF_SIZE, "Can't get Bluetooth device list via ioctl %d:%s",
-			errno, strerror(errno));
+		snprintf(err_str, PCAP_ERRBUF_SIZE,
+		    "Can't get Bluetooth device list via ioctl: %s",
+		    strerror(errno));
 		ret = -1;
 		goto free;
 	}
@@ -134,10 +135,40 @@ done:
 }
 
 pcap_t *
-bt_create(const char *device, char *ebuf)
+bt_create(const char *device, char *ebuf, int *is_ours)
 {
+	const char *cp;
+	char *cpend;
+	long devnum;
 	pcap_t *p;
 
+	/* Does this look like a Bluetooth device? */
+	cp = strrchr(device, '/');
+	if (cp == NULL)
+		cp = device;
+	/* Does it begin with BT_IFACE? */
+	if (strncmp(cp, BT_IFACE, sizeof BT_IFACE - 1) != 0) {
+		/* Nope, doesn't begin with BT_IFACE */
+		*is_ours = 0;
+		return NULL;
+	}
+	/* Yes - is BT_IFACE followed by a number? */
+	cp += sizeof BT_IFACE - 1;
+	devnum = strtol(cp, &cpend, 10);
+	if (cpend == cp || *cpend != '\0') {
+		/* Not followed by a number. */
+		*is_ours = 0;
+		return NULL;
+	}
+	if (devnum < 0) {
+		/* Followed by a non-valid number. */
+		*is_ours = 0;
+		return NULL;
+	}
+
+	/* OK, it's probably ours. */
+	*is_ours = 1;
+
 	p = pcap_create_common(device, ebuf);
 	if (p == NULL)
 		return (NULL);
@@ -182,8 +213,8 @@ bt_activate(pcap_t* handle)
 	/* Create HCI socket */
 	handle->fd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (handle->fd < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't create raw socket %d:%s",
-			errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't create raw socket: %s", strerror(errno));
 		return PCAP_ERROR;
 	}
 
@@ -196,15 +227,15 @@ bt_activate(pcap_t* handle)
 
 	opt = 1;
 	if (setsockopt(handle->fd, SOL_HCI, HCI_DATA_DIR, &opt, sizeof(opt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't enable data direction info %d:%s",
-			errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't enable data direction info: %s", strerror(errno));
 		goto close_fail;
 	}
 
 	opt = 1;
 	if (setsockopt(handle->fd, SOL_HCI, HCI_TIME_STAMP, &opt, sizeof(opt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't enable time stamp %d:%s",
-			errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't enable time stamp: %s", strerror(errno));
 		goto close_fail;
 	}
 
@@ -214,8 +245,8 @@ bt_activate(pcap_t* handle)
 	memset((void *) &flt.type_mask, 0xff, sizeof(flt.type_mask));	
 	memset((void *) &flt.event_mask, 0xff, sizeof(flt.event_mask));
 	if (setsockopt(handle->fd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't set filter %d:%s",
-			errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't set filter: %s", strerror(errno));
 		goto close_fail;
 	}
 
@@ -223,9 +254,13 @@ bt_activate(pcap_t* handle)
 	/* Bind socket to the HCI device */
 	addr.hci_family = AF_BLUETOOTH;
 	addr.hci_dev = handle->md.ifindex;
+#ifdef SOCKADDR_HCI_HAS_HCI_CHANNEL
+	addr.hci_channel = HCI_CHANNEL_RAW;
+#endif
 	if (bind(handle->fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't attach to device %d %d:%s",
-			handle->md.ifindex, errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't attach to device %d: %s", handle->md.ifindex,
+		    strerror(errno));
 		goto close_fail;
 	}
 
@@ -264,6 +299,7 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 	struct cmsghdr *cmsg;
 	struct msghdr msg;
 	struct iovec  iv;
+	ssize_t ret;
 	struct pcap_pkthdr pkth;
 	pcap_bluetooth_h4_header* bthdr;
 
@@ -279,21 +315,22 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 
 	/* ignore interrupt system call error */
 	do {
-		pkth.caplen = recvmsg(handle->fd, &msg, 0);
+		ret = recvmsg(handle->fd, &msg, 0);
 		if (handle->break_loop)
 		{
 			handle->break_loop = 0;
 			return -2;
 		}
-	} while ((pkth.caplen == -1) && (errno == EINTR));
+	} while ((ret == -1) && (errno == EINTR));
 
-		
-	if (pkth.caplen < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't receive packet %d:%s",
-			errno, strerror(errno));
+	if (ret < 0) {
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't receive packet: %s", strerror(errno));
 		return -1;
 	}
 
+	pkth.caplen = ret;
+
 	/* get direction and timestamp*/ 
 	cmsg = CMSG_FIRSTHDR(&msg);
 	int in=0;
@@ -348,8 +385,8 @@ bt_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 	} while ((ret == -1) && (errno == EINTR));
 	    
 	if (ret < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "can get stats"
-			" via ioctl %d:%s", errno, strerror(errno));
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		    "Can't get stats via ioctl: %s", strerror(errno));
 		return (-1);
 		
 	}
