@@ -389,11 +389,12 @@ pcap_activate_snoop(pcap_t *p)
 
 	return (0);
  bad:
+	pcap_cleanup_live_common(p);
 	return (PCAP_ERROR);
 }
 
 pcap_t *
-pcap_create(const char *device, char *ebuf)
+pcap_create_interface(const char *device, char *ebuf)
 {
 	pcap_t *p;
 
